/*--------------------------------------------------- 2017-12-08 ------*/
/* Signal start                                                        */
/*---------------------------------------------------------------------*/
#include <MsTimer2.h>
#include <SoftwareSerial.h>
#include <DFPlayer_Mini_Mp3.h>

SoftwareSerial mySerial(11, 12); // RX, TX

#define RELAY_ON   LOW
#define RELAY_OFF  HIGH

#define SWITCH_ON  LOW
#define SWITCH_OFF HIGH

#define SOUND_ON   LOW
#define SOUND_OFF  HIGH

#define OUT_ON     HIGH
#define OUT_OFF    LOW

#define SOUND_ON_TIME  100   // msec
#define LAMP_ON_TIME   400   // msec
#define LAMP_OFF_TIME  500   // msec

#define LAMP_ON_TIME_FALSE_STRAT   200   // msec
#define LAMP_OFF_TIME_FALSE_STRAT  300   // msec

//----- port output -------------
#define RELAY_RED_PORT           7   // digital port 7
#define RELAY_YELLOW_PORT        6   // digital port 6
#define RELAY_GREEN_PORT         5   // digital port 5
#define RELAY4_PORT              4   // digital port 4
#define SOUND_RED_PORT           8   // digital port 8
#define SOUND_GREEN_PORT         9   // digital port 9
#define SOUND_FALSE_START_PORT  10   // digital port 10
#define OUT_GREEN_PORT          13   // digital port 13

//----- port input -------------
#define FALSE_START_PORT         2  // digital port 2
#define START_SWITCH_PORT       A1  // analog port 1
#define ROTARY_SWITCH_0_PORT    A2  // analog port 2 
#define ROTARY_SWITCH_1_PORT    A3  // analog port 3
#define ROTARY_SWITCH_2_PORT    A4  // analog port 4
#define ROTARY_SWITCH_3_PORT    A5  // analog port 5

#define SEQUENCE_START        1
#define SEQUENCE_FALSE_START  2

boolean  switch_enable;

byte Relay_RED;
byte Relay_YELLOW;
byte Relay_GREEN;
byte Out_GREEN;

byte Relay_RED_FS;
byte Relay_YELLOW_FS;

byte    wait_counter_start;
byte    wait_counter_false_start;
boolean tick_start;
boolean tick_false_start;

boolean sequence_start;
boolean sequence_false_start;
byte sequence_index_start;
byte sequence_index_false_start;
byte false_start_loop_counter;

byte sequence_array[][2] = {
  B10010000, 25,    //  start sequence begin   index 0
  B10000000, 25,
  B10010000, 25,
  B10000000, 25,
  B01010000, 25,
  B01000000, 25,
  B00101010, 5,
  B00000010, 250,
  B00100000, 5,
  B00000000, 0,  
  B11100100,10,  // false sequence start    index 10
  B11100000, 5,
  B00000000, 0   
};

#define FALSE_START_INDEX  10

//--------------------------------------------------------------------------
// all off
//--------------------------------------------------------------------------
void all_off( byte seq_type ) {
  switch ( seq_type ) {
    case SEQUENCE_START:
      Relay_RED         = RELAY_OFF;
      Relay_YELLOW      = RELAY_OFF;
      Relay_GREEN       = RELAY_OFF;
      digitalWrite(RELAY_RED_PORT,         Relay_RED    );
      digitalWrite(RELAY_YELLOW_PORT,      Relay_YELLOW );
      digitalWrite(RELAY_GREEN_PORT,       Relay_GREEN  );
      break;
    case SEQUENCE_FALSE_START:
      Relay_RED_FS         = RELAY_OFF;
      Relay_YELLOW_FS      = RELAY_OFF;
      digitalWrite(RELAY_RED_PORT,         Relay_RED_FS         );
      digitalWrite(RELAY_YELLOW_PORT,      Relay_YELLOW_FS      );
      break;
  }
}

//--------------------------------------------------------------------------
// false start
//--------------------------------------------------------------------------
void false_start() {
  sequence_false_start = true;
  sequence_index_false_start = FALSE_START_INDEX;
  switch_enable = false;
  false_start_loop_counter = 11;
}

//--------------------------------------------------------------------------
// execute start sequence
//--------------------------------------------------------------------------
boolean execute_sequence( byte sequence_index, byte sequence_type ) {
  switch ( sequence_type ) {
    case SEQUENCE_START:
      if ( sequence_array[sequence_index][0] == 0 ) {
        return false;
      }
      if ( (B10000000 & sequence_array[sequence_index][0]) != 0 ) {
        if ( Relay_RED    == RELAY_ON ) {
          Relay_RED    = RELAY_OFF;
        } else {
          Relay_RED    = RELAY_ON;
        }
      }
      if ( (B01000000 & sequence_array[sequence_index][0]) != 0 ) {
        if ( Relay_YELLOW == RELAY_ON ) {
          Relay_YELLOW = RELAY_OFF;
        } else {
          Relay_YELLOW = RELAY_ON;
        }
      }
      if ( (B00100000 & sequence_array[sequence_index][0]) != 0 ) {
        if ( Relay_GREEN  == RELAY_ON ) {
          Relay_GREEN  = RELAY_OFF;
        } else {
          Relay_GREEN  = RELAY_ON;
        }
      }
      if ( (B00010000 & sequence_array[sequence_index][0]) != 0 ) {
        if( sequence_false_start == false ) {
          mp3_play (1);
        }
      }
      if ( (B00001000 & sequence_array[sequence_index][0]) != 0 ) {
        if( sequence_false_start == false ) {
          mp3_play (2);
        }
      }
      if ( (B00000010 & sequence_array[sequence_index][0]) != 0 ) {
        if ( Out_GREEN == OUT_ON ) {
          Out_GREEN = OUT_OFF;
        } else {
          Out_GREEN = OUT_ON;
        }
      }
      if ( sequence_false_start == false ) {
        digitalWrite(RELAY_RED_PORT,    Relay_RED    );
        digitalWrite(RELAY_YELLOW_PORT, Relay_YELLOW );
      }
      digitalWrite(RELAY_GREEN_PORT, Relay_GREEN );
      digitalWrite(OUT_GREEN_PORT,   Out_GREEN   );

      wait_counter_start = sequence_array[sequence_index][1];
      return true;
    case SEQUENCE_FALSE_START:
      if ( sequence_array[sequence_index][0] == 0 ) {
        return false;
      }
      if ( (B10000000 & sequence_array[sequence_index][0]) != 0 ) {
        if ( Relay_RED_FS    == RELAY_ON ) {
          Relay_RED_FS    = RELAY_OFF;
        } else {
          Relay_RED_FS    = RELAY_ON;
        }
      }
      if ( (B01000000 & sequence_array[sequence_index][0]) != 0 ) {
        if ( Relay_YELLOW_FS == RELAY_ON ) {
          Relay_YELLOW_FS = RELAY_OFF;
        } else {
          Relay_YELLOW_FS = RELAY_ON;
        }
      }
      if ( (B00000100 & sequence_array[sequence_index][0]) != 0 ) {
        mp3_play(3);
      }
      digitalWrite(RELAY_RED_PORT,         Relay_RED_FS          );
      digitalWrite(RELAY_YELLOW_PORT,      Relay_YELLOW_FS       );

      wait_counter_false_start = sequence_array[sequence_index][1];
      return true;
  }
}

//--------------------------------------------------------------------------
// false start
//--------------------------------------------------------------------------
void elapsed_20msec() {
  if ( wait_counter_start > 0  ) {
    wait_counter_start--;
  } else {
    tick_start = true;
  }

  if ( wait_counter_false_start > 0  ) {
    wait_counter_false_start--;
  } else {
    tick_false_start = true;
  }
}

//--------------------------------------------------------------------------
// arduino saetup
//--------------------------------------------------------------------------
void setup()   {
  pinMode(RELAY_RED_PORT,         OUTPUT);
  pinMode(RELAY_YELLOW_PORT,      OUTPUT);
  pinMode(RELAY_GREEN_PORT,       OUTPUT);
  pinMode(RELAY4_PORT,            OUTPUT);
  pinMode(OUT_GREEN_PORT,         OUTPUT);

  digitalWrite(RELAY4_PORT, RELAY_OFF);

  all_off( SEQUENCE_START       );
  all_off( SEQUENCE_FALSE_START );
  Out_GREEN = OUT_OFF;
  digitalWrite(OUT_GREEN_PORT, Out_GREEN );

  pinMode(FALSE_START_PORT,     INPUT);
  pinMode(START_SWITCH_PORT,    INPUT);
  pinMode(ROTARY_SWITCH_0_PORT, INPUT);
  pinMode(ROTARY_SWITCH_1_PORT, INPUT);
  pinMode(ROTARY_SWITCH_2_PORT, INPUT);
  pinMode(ROTARY_SWITCH_3_PORT, INPUT);

  attachInterrupt(0, false_start, RISING);  // on digital pin 2

  switch_enable = true;
  wait_counter_start = 0;
  wait_counter_false_start = 0;

  MsTimer2::set(20, elapsed_20msec);     // 20msec period
  MsTimer2::start();

  sequence_start       = false;
  sequence_false_start = false;
  
  mySerial.begin(9600);
  mp3_set_serial(mySerial);  //set softwareSerial for DFPlayer-mini mp3 module 
  delay(1000);  //wait 1ms for mp3 module to set volume
  mp3_set_volume(30);  
}

//--------------------------------------------------------------------------
// arduino loop
//--------------------------------------------------------------------------
void loop()
{
  if ( switch_enable ) {
    if (  digitalRead(START_SWITCH_PORT) == SWITCH_ON ) {
      switch_enable = false;
      digitalWrite( OUT_GREEN_PORT, HIGH );
      delay( 100 );
      digitalWrite( OUT_GREEN_PORT, LOW );
      if( digitalRead(ROTARY_SWITCH_0_PORT) == SWITCH_ON ) { delay(  900 ); }
      if( digitalRead(ROTARY_SWITCH_1_PORT) == SWITCH_ON ) { delay( 1900 ); }
      if( digitalRead(ROTARY_SWITCH_2_PORT) == SWITCH_ON ) { delay( 2900 ); }
      if( digitalRead(ROTARY_SWITCH_3_PORT) == SWITCH_ON ) { delay( 3900 ); }
      sequence_start = true;
      sequence_index_start = 0;
      switch_enable = false;
      all_off( SEQUENCE_START );
    }
  }

  if ( tick_start ) {
    //---- start sequence -----------------------------
    if ( sequence_start ) {
      if ( execute_sequence( sequence_index_start, SEQUENCE_START ) == false ) {
        all_off( SEQUENCE_START );
        switch_enable = true;
        sequence_start = false;
      } else {
        sequence_index_start++;
      }
    }
    tick_start = false;
  }

  if ( tick_false_start ) {
    //---- false start sequence -----------------------
    if ( sequence_false_start ) {
      if ( execute_sequence( sequence_index_false_start, SEQUENCE_FALSE_START ) == false ) {
        if ( false_start_loop_counter > 0 ) {
          sequence_index_false_start = FALSE_START_INDEX;
          false_start_loop_counter--;
        } else {
          all_off( SEQUENCE_FALSE_START );
          sequence_false_start = false;
          switch_enable = true;
        }
      } else {
        sequence_index_false_start++;
      }
    }
    tick_false_start = false;
  }
}

//-------------- E N D -------------------------------------------------------------
